@@ -9,26 +9,32 @@ and SDK layer, providing a clean interface for the view layer.
 """
 
 import asyncio
-from typing import Any, Dict, Optional
+from typing import Any
 
 from gi.repository import GLib, GObject
 
 from waydroid_helper.util import Task, logger
-from waydroid_helper.models import PropertyCategory, PropertyModel, SessionModel, ModelState, SessionState
+from waydroid_helper.models import (
+    PropertyCategory,
+    PropertyModel,
+    SessionModel,
+    ModelState,
+    SessionState,
+)
 from waydroid_helper.sdk import WaydroidSDK, PropertyManager, ConfigManager
 
 
 class ModelController(GObject.Object):
     """
     Controller that coordinates between models and SDK.
-    
+
     This controller:
     - Manages the lifecycle of models
     - Coordinates operations between models and SDK
     - Provides a clean interface for the view layer
     - Handles async operations and error handling
     """
-    
+
     def __init__(self):
         super().__init__()
 
@@ -48,7 +54,7 @@ class ModelController(GObject.Object):
 
         # Defer async initialization until event loop is available
         GLib.idle_add(self._start_status_monitoring)
-    
+
     def _start_status_monitoring(self):
         """Start periodic monitoring of Waydroid status"""
         if self._monitoring_started:
@@ -62,6 +68,9 @@ class ModelController(GObject.Object):
         # Schedule periodic updates every 2 seconds
         GLib.timeout_add_seconds(2, self._schedule_status_update)
 
+        # Schedule periodic ERROR state recovery checks every 10 seconds
+        GLib.timeout_add_seconds(10, self._schedule_error_recovery)
+
         return False  # Don't repeat this idle callback
 
     async def _initial_status_check(self):
@@ -76,15 +85,12 @@ class ModelController(GObject.Object):
             if current_state in (SessionState.STOPPED, SessionState.RUNNING):
                 await self._load_privileged_properties()
                 await self._load_waydroid_properties()
+                await self._load_android_version()
 
             # Load persist properties if session is running
             if current_state == SessionState.RUNNING:
                 await self._load_persist_properties()
 
-            # Load Android version if waydroid is initialized
-            if current_state in (SessionState.STOPPED, SessionState.RUNNING):
-                await self._load_android_version()
-
         except Exception as e:
             logger.error(f"Failed initial status check: {e}")
 
@@ -92,7 +98,12 @@ class ModelController(GObject.Object):
         """Schedule a status update task"""
         self._task.create_task(self._update_session_status())
         return True  # Continue the timeout
-    
+
+    def _schedule_error_recovery(self) -> bool:
+        """Schedule an error recovery check"""
+        self._task.create_task(self._handle_error_state_recovery())
+        return True  # Continue the timeout
+
     async def _update_session_status(self):
         """Update session status and load properties if needed"""
         async with self._status_update_lock:
@@ -102,18 +113,24 @@ class ModelController(GObject.Object):
 
                 # Only log significant state changes
                 if old_state != new_state:
-                    logger.debug(f"Session state transition: {old_state} -> {new_state}")
+                    logger.debug(
+                        f"Session state transition: {old_state} -> {new_state}"
+                    )
 
                 # Update session state
                 self.session_model.set_session_state(new_state)
 
+                _ = await self.refresh_persist_property("boot-completed")
+
                 # Handle state transitions
                 await self._handle_session_state_transition(old_state, new_state)
 
             except Exception as e:
                 logger.error(f"Failed to update session status: {e}")
-    
-    async def _handle_session_state_transition(self, old_state: SessionState, new_state: SessionState):
+
+    async def _handle_session_state_transition(
+        self, old_state: SessionState, new_state: SessionState
+    ):
         """Handle session state transitions and load/unload properties accordingly"""
 
         # When session becomes running, load persist properties
@@ -122,6 +139,8 @@ class ModelController(GObject.Object):
             # Also load Android version if coming from UNINITIALIZED
             if old_state == SessionState.UNINITIALIZED:
                 await self._load_android_version()
+                await self._load_privileged_properties()
+                await self._load_waydroid_properties()
 
         # When session stops, reset persist props state but keep privileged props
         elif new_state == SessionState.STOPPED and old_state == SessionState.RUNNING:
@@ -129,34 +148,36 @@ class ModelController(GObject.Object):
             # privileged_state remains READY
 
         # When Waydroid becomes initialized (stopped), load privileged and waydroid properties
-        elif new_state == SessionState.STOPPED and old_state == SessionState.UNINITIALIZED:
-            await self._load_privileged_properties()
-            await self._load_waydroid_properties()
+        elif (
+            new_state == SessionState.STOPPED
+            and old_state == SessionState.UNINITIALIZED
+        ):
+            await self._load_privileged_properties_with_retry()
+            await self._load_waydroid_properties_with_retry()
             await self._load_android_version()
 
-        # When waydroid becomes coproperty_modelmpletely uninitialized, reset all states
+        # When waydroid becomes completely uninitialized, reset all states
         elif new_state == SessionState.UNINITIALIZED:
             self.property_model.set_property("state", ModelState.UNINITIALIZED)
-            self.property_model.set_property("privileged-state", ModelState.UNINITIALIZED)
+            self.property_model.set_property(
+                "privileged-state", ModelState.UNINITIALIZED
+            )
             self.property_model.set_property("waydroid-state", ModelState.UNINITIALIZED)
-    
 
-    
+        # Handle ERROR state recovery - retry loading if states are in ERROR
+        await self._handle_error_state_recovery()
+
     async def _load_persist_properties(self):
         """Load persist properties from Waydroid"""
         self.property_model.set_property("state", ModelState.LOADING)
 
         try:
-            property_definitions = self.property_model._property_definitions
             property_values = await self.property_manager.get_all_persist_properties(
-                property_definitions
+                self.property_model.get_persist_properties()
             )
 
             for prop_name, raw_value in property_values.items():
-                prop_def = self.property_model.get_property_definition(prop_name)
-                if prop_def:
-                    transformed_value = prop_def.transform_in(raw_value)
-                    self.property_model.set_property_value(prop_name, transformed_value)
+                self.property_model.set_property_raw_value(prop_name, raw_value)
 
             self.property_model.set_property("state", ModelState.READY)
 
@@ -164,7 +185,7 @@ class ModelController(GObject.Object):
             logger.error(f"Failed to load persist properties: {e}")
             self.property_model.set_property("state", ModelState.ERROR)
             raise
-    
+
     async def _load_privileged_properties(self):
         """Load privileged properties from config"""
         self.property_model.set_property("privileged-state", ModelState.LOADING)
@@ -174,21 +195,18 @@ class ModelController(GObject.Object):
                 raise Exception("Failed to load config")
 
             property_values = self.config_manager.get_all_privileged_properties(
-                self.property_model._property_definitions
+                self.property_model.get_privileged_properties()
             )
 
             for prop_name, raw_value in property_values.items():
-                prop_def = self.property_model.get_property_definition(prop_name)
-                if prop_def:
-                    transformed_value = prop_def.transform_in(raw_value)
-                    self.property_model.set_property_value(prop_name, transformed_value)
+                self.property_model.set_property_raw_value(prop_name, raw_value)
 
             self.property_model.set_property("privileged-state", ModelState.READY)
 
         except Exception as e:
             logger.error(f"Failed to load privileged properties: {e}")
             self.property_model.set_property("privileged-state", ModelState.ERROR)
-            raise
+            # Don't raise - allow other operations to continue
 
     async def _load_waydroid_properties(self):
         """Load waydroid config properties from [waydroid] section"""
@@ -199,21 +217,68 @@ class ModelController(GObject.Object):
                 raise Exception("Failed to load config")
 
             property_values = self.config_manager.get_all_waydroid_properties(
-                self.property_model._property_definitions
+                self.property_model.get_waydroid_properties()
             )
 
             for prop_name, raw_value in property_values.items():
-                prop_def = self.property_model.get_property_definition(prop_name)
-                if prop_def:
-                    transformed_value = prop_def.transform_in(raw_value)
-                    self.property_model.set_property_value(prop_name, transformed_value)
+                self.property_model.set_property_raw_value(prop_name, raw_value)
 
             self.property_model.set_property("waydroid-state", ModelState.READY)
 
         except Exception as e:
             logger.error(f"Failed to load waydroid properties: {e}")
             self.property_model.set_property("waydroid-state", ModelState.ERROR)
-            raise
+            # Don't raise - allow other operations to continue
+
+    async def _load_privileged_properties_with_retry(self, max_retries: int = 3):
+        """Load privileged properties with retry mechanism"""
+        for attempt in range(max_retries):
+            try:
+                await self._load_privileged_properties()
+                return  # Success, exit retry loop
+            except Exception as e:
+                logger.warning(
+                    f"Attempt {attempt + 1}/{max_retries} to load privileged properties failed: {e}"
+                )
+                if attempt < max_retries - 1:
+                    # Wait before retry (exponential backoff)
+                    await asyncio.sleep(2**attempt)
+                else:
+                    logger.error(
+                        f"Failed to load privileged properties after {max_retries} attempts"
+                    )
+
+    async def _load_waydroid_properties_with_retry(self, max_retries: int = 3):
+        """Load waydroid properties with retry mechanism"""
+        for attempt in range(max_retries):
+            try:
+                await self._load_waydroid_properties()
+                return  # Success, exit retry loop
+            except Exception as e:
+                logger.warning(
+                    f"Attempt {attempt + 1}/{max_retries} to load waydroid properties failed: {e}"
+                )
+                if attempt < max_retries - 1:
+                    # Wait before retry (exponential backoff)
+                    await asyncio.sleep(2**attempt)
+                else:
+                    logger.error(
+                        f"Failed to load waydroid properties after {max_retries} attempts"
+                    )
+
+    async def _handle_error_state_recovery(self):
+        """Handle recovery from ERROR states by retrying failed operations"""
+        # Check if privileged properties are in ERROR state and retry
+        privileged_state = self.property_model.get_property("privileged-state")
+        if privileged_state == ModelState.ERROR:
+            logger.info("Attempting to recover from privileged properties ERROR state")
+            await self._load_privileged_properties_with_retry(max_retries=2)
+
+        # Check if waydroid properties are in ERROR state and retry
+        waydroid_state = self.property_model.get_property("waydroid-state")
+        if waydroid_state == ModelState.ERROR:
+            logger.info("Attempting to recover from waydroid properties ERROR state")
+            await self._load_waydroid_properties_with_retry(max_retries=2)
 
     async def _load_android_version(self):
         """Load Android version from config"""
@@ -222,25 +287,17 @@ class ModelController(GObject.Object):
             self.session_model.set_property("android_version", android_version)
         except Exception as e:
             logger.error(f"Failed to load Android version: {e}")
-    
+
     # Public API for view layer
-    
+
     async def save_persist_property(self, property_name: str) -> bool:
         """Save a single persist property"""
         try:
-            # Convert property name format (handle both dashes and underscores)
-            normalized_name = property_name.replace("-", "_")
+            raw_value = self.property_model.get_property_raw_value(property_name)
+            nick = self.property_model.find_property(property_name).get_nick()
 
-            prop_def = self.property_model.get_property_definition(normalized_name)
-            if not prop_def or prop_def.category != PropertyCategory.PERSIST:
-                logger.error(f"Property {property_name} (normalized: {normalized_name}) is not a persist property")
-                return False
-
-            value = self.property_model.get_property_value(normalized_name)
-            transformed_value = prop_def.transform_out(value)
-
-            logger.info(f"Saving persist property {normalized_name} = {value} (transformed: {transformed_value})")
-            return await self.property_manager.set_persist_property(prop_def.nick, transformed_value)
+            logger.info(f"Saved persist property {property_name} = {raw_value}")
+            return await self.property_manager.set_persist_property(nick, raw_value)
 
         except Exception as e:
             logger.error(f"Failed to save persist property {property_name}: {e}")
@@ -249,28 +306,19 @@ class ModelController(GObject.Object):
     async def refresh_persist_property(self, property_name: str) -> bool:
         """Refresh a single persist property from Waydroid"""
         try:
-            # Convert property name format (handle both dashes and underscores)
-            normalized_name = property_name.replace("-", "_")
-
-            prop_def = self.property_model.get_property_definition(normalized_name)
-            if not prop_def or prop_def.category != PropertyCategory.PERSIST:
-                logger.error(f"Property {property_name} (normalized: {normalized_name}) is not a persist property")
-                return False
-
             # Get current value from Waydroid
-            raw_value = await self.property_manager.get_persist_property(prop_def.nick)
-            transformed_value = prop_def.transform_in(raw_value)
+            nick = self.property_model.find_property(property_name).get_nick()
+            raw_value = await self.property_manager.get_persist_property(nick)
 
-            # Update the model
-            self.property_model.set_property_value(normalized_name, transformed_value)
+            self.property_model.set_property_raw_value(property_name, raw_value)
 
-            logger.info(f"Refreshed persist property {normalized_name} = {transformed_value} (raw: {raw_value})")
+            logger.debug(f"Refreshed persist property {property_name} = {raw_value}")
             return True
 
         except Exception as e:
             logger.error(f"Failed to refresh persist property {property_name}: {e}")
             return False
-    
+
     async def save_all_privileged_properties(self) -> bool:
         """Save all privileged properties to config file"""
         try:
@@ -278,10 +326,10 @@ class ModelController(GObject.Object):
             privileged_props = self.property_model.get_privileged_properties()
             properties_to_save = {}
 
-            for prop_name, prop_def in privileged_props.items():
-                value = self.property_model.get_property_value(prop_name)
-                transformed_value = prop_def.transform_out(value)
-                properties_to_save[prop_def.nick] = transformed_value
+            for prop in privileged_props:
+                nick = prop.get_nick()
+                raw_value = self.property_model.get_property_raw_value(prop.get_name())
+                properties_to_save[nick] = raw_value
 
             # Set all properties in config
             self.config_manager.set_multiple_privileged_properties(properties_to_save)
@@ -289,53 +337,62 @@ class ModelController(GObject.Object):
             # Save config file
             success = await self.config_manager.save_config()
 
-            if success:
-                # Trigger upgrade to apply changes
-                await self.waydroid_sdk.upgrade(offline=True)
-
             return success
-            
+
         except Exception as e:
             logger.error(f"Failed to save privileged properties: {e}")
             return False
-    
+
     async def reset_persist_properties(self) -> bool:
         """Reset all persist properties to defaults"""
+        self.property_model.set_property("state", ModelState.LOADING)
         try:
             persist_props = self.property_model.get_persist_properties()
-            
+
             # Reset in model
             self.property_model.reset_to_defaults(PropertyCategory.PERSIST)
-            
+
             # Save all persist properties
             tasks = []
-            for prop_name in persist_props.keys():
-                task = self.save_persist_property(prop_name)
+            for prop in persist_props:
+                task = self.save_persist_property(prop.get_name())
                 tasks.append(task)
-            
+
             results = await asyncio.gather(*tasks, return_exceptions=True)
-            
+            self.property_model.set_property("state", ModelState.READY)
+
             # Check if all succeeded
-            return all(result is True for result in results if not isinstance(result, Exception))
-            
+            return all(
+                result is True
+                for result in results
+                if not isinstance(result, Exception)
+            )
+
         except Exception as e:
             logger.error(f"Failed to reset persist properties: {e}")
+            self.property_model.set_property("state", ModelState.ERROR)
             return False
-    
+
     async def reset_privileged_properties(self) -> bool:
         """Reset all privileged properties to defaults"""
+        self.property_model.set_property("privileged-state", ModelState.LOADING)
         try:
-            # Reset in model
             self.property_model.reset_to_defaults(PropertyCategory.PRIVILEGED)
-            
-            # Reset in config
-            self.config_manager.reset_privileged_properties()
-            
-            # Save config
-            return await self.config_manager.save_config()
-            
+
+            props = self.property_model.get_privileged_properties()
+            for prop in props:
+                self.config_manager.set_privileged_property(prop.get_nick(), "")
+
+            success = await self.config_manager.save_config()
+            if success:
+                return await self.upgrade(offline=True)
+
+            self.property_model.set_property("privileged-state", ModelState.READY)
+            return success
+
         except Exception as e:
             logger.error(f"Failed to reset privileged properties: {e}")
+            self.property_model.set_property("privileged-state", ModelState.ERROR)
             return False
 
     async def save_all_waydroid_properties(self) -> bool:
@@ -345,10 +402,10 @@ class ModelController(GObject.Object):
             waydroid_props = self.property_model.get_waydroid_properties()
             properties_to_save = {}
 
-            for prop_name, prop_def in waydroid_props.items():
-                value = self.property_model.get_property_value(prop_name)
-                transformed_value = prop_def.transform_out(value)
-                properties_to_save[prop_def.nick] = transformed_value
+            for prop in waydroid_props:
+                nick = prop.get_nick()
+                raw_value = self.property_model.get_property_raw_value(prop.get_name())
+                properties_to_save[nick] = raw_value
 
             # Set all properties in config
             self.config_manager.set_multiple_waydroid_properties(properties_to_save)
@@ -357,7 +414,7 @@ class ModelController(GObject.Object):
             success = await self.config_manager.save_config()
 
             if success:
-                # Trigger upgrade to apply changes
+                # Trigger restart container to apply changes
                 await self.waydroid_sdk.stop_session(wait=True)
                 await self.waydroid_sdk.restart_container(wait=True)
 
@@ -369,118 +426,132 @@ class ModelController(GObject.Object):
 
     async def reset_waydroid_properties(self) -> bool:
         """Reset all waydroid config properties to defaults"""
+        self.property_model.set_property("waydroid-state", ModelState.LOADING)
         try:
-            # Reset in model
             self.property_model.reset_to_defaults(PropertyCategory.WAYDROID)
 
-            # Reset in config
-            self.config_manager.reset_waydroid_properties(self.property_model._property_definitions)
+            props = self.property_model.get_waydroid_properties()
+            for prop in props:
+                nick = prop.get_nick()
+                raw_value = self.property_model.get_property_raw_value(prop.get_name())
+                self.config_manager.set_waydroid_property(nick, raw_value)
 
-            # Save config
-            return await self.config_manager.save_config()
+            success = await self.config_manager.save_config()
+            if success:
+                return await self.upgrade(offline=True)
+
+            self.property_model.set_property("waydroid-state", ModelState.READY)
+            return success
 
         except Exception as e:
             logger.error(f"Failed to reset waydroid properties: {e}")
+            self.property_model.set_property("waydroid-state", ModelState.ERROR)
             return False
 
     async def restore_privileged_properties(self) -> bool:
         """Restore privileged properties from saved config"""
+        self.property_model.set_property("privileged-state", ModelState.LOADING)
         try:
             # Reload config from file
             if not self.config_manager.load_config():
                 return False
-            
+
             # Reload properties into model
             await self._load_privileged_properties()
-            
+
+            self.property_model.set_property("privileged-state", ModelState.READY)
             return True
-            
+
         except Exception as e:
             logger.error(f"Failed to restore privileged properties: {e}")
+            self.property_model.set_property("privileged-state", ModelState.ERROR)
             return False
-    
+
     async def restore_waydroid_properties(self) -> bool:
         """Restore waydroid config properties from saved config"""
+        self.property_model.set_property("waydroid-state", ModelState.LOADING)
         try:
             # Reload config from file
             if not self.config_manager.load_config():
+                self.property_model.set_property("privileged-state", ModelState.ERROR)
                 return False
-            
+
             # Reload properties into model
             await self._load_waydroid_properties()
-            
+
+            self.property_model.set_property("waydroid-state", ModelState.READY)
             return True
-            
+
         except Exception as e:
             logger.error(f"Failed to restore waydroid properties: {e}")
+            self.property_model.set_property("waydroid-state", ModelState.ERROR)
             return False
-    
-    async def set_device_info(self, device_properties: Dict[str, Any]) -> bool:
+
+    async def set_device_info(self, device_properties: dict[str, Any]) -> bool:
         """Set device information properties"""
         try:
             # Update model
             for prop_name, value in device_properties.items():
                 # Convert property names (e.g., "ro.product.brand" -> "ro_product_brand")
                 model_prop_name = prop_name.replace(".", "_")
-                self.property_model.set_property_value(model_prop_name, value)
-            self.config_manager.set_multiple_privileged_properties(device_properties)        
+                _ = self.property_model.set_property_raw_value(model_prop_name, value)
+            self.config_manager.set_multiple_privileged_properties(device_properties)
             return True
-            
+
         except Exception as e:
             logger.error(f"Failed to set device info: {e}")
             return False
-    
-    async def set_extension_properties(self, properties: Dict[str, Any]) -> bool:
+
+    async def set_extension_properties(self, properties: dict[str, Any]) -> bool:
         """Set extension properties"""
         try:
             # Set properties in config
             self.config_manager.set_multiple_privileged_properties(properties)
-            
-            # Save and upgrade
+
             success = await self.config_manager.save_config()
-            
+
             return success
-            
+
         except Exception as e:
             logger.error(f"Failed to set extension properties: {e}")
             return False
-    
+
     async def remove_extension_properties(self, property_keys: list[str]) -> bool:
         """Remove extension properties"""
         try:
             # Remove properties from config
             empty_properties = {key: "" for key in property_keys}
             self.config_manager.set_multiple_privileged_properties(empty_properties)
-            
-            # Save and upgrade
+
             success = await self.config_manager.save_config()
-            if success:
-                await self.waydroid_sdk.upgrade(offline=True)
-            
+
             return success
-            
+
         except Exception as e:
             logger.error(f"Failed to remove extension properties: {e}")
             return False
-    
+
     # Session management
-    
+
     async def start_session(self) -> bool:
         """Start Waydroid session"""
         return await self.waydroid_sdk.start_session()
-    
+
     async def stop_session(self) -> bool:
         """Stop Waydroid session"""
         return await self.waydroid_sdk.stop_session()
-    
+
     async def restart_session(self) -> bool:
         """Restart Waydroid session"""
         return await self.waydroid_sdk.restart_session()
-    
+
     async def show_full_ui(self) -> bool:
         """Show Waydroid full UI"""
         return await self.waydroid_sdk.show_full_ui()
-    
+
     async def upgrade(self, offline: bool = False) -> bool:
         """Upgrade Waydroid"""
-        return await self.waydroid_sdk.upgrade(offline)
+        success = await self.waydroid_sdk.upgrade(offline)
+        await self._load_privileged_properties_with_retry()
+        await self._load_waydroid_properties_with_retry()
+        return success
